package de.howaner.MapEditor.gui;

import de.howaner.MapEditor.MainMapEditor;
import de.howaner.MapEditor.gui.components.*;
import de.howaner.MapEditor.map.MapWalkData;
import de.howaner.MapEditor.map.events.Event;
import de.howaner.MapEditor.map.events.EventType;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class MyWindow extends JFrame implements MouseWheelListener, ChangeListener {
	private final JTabbedPane tabbedPane;

	private GuiTilesetList tilesetList;
	private GuiWalkDataList walkDataList;
	private GuiImageMap imageMap;
	private JScrollPane mapScrollPane;

	private JLabel selectedEventTypeLabel;
	private GuiEventTypesList eventTypesList;

	private Box eventBox;
	
	public MyWindow(MainMapEditor main) {
		super("Map Editor by Howaner");
		this.setPreferredSize(new Dimension(800, 600));
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setJMenuBar(new GuiMenuBar());
		this.getContentPane().setLayout(new BoxLayout(this.getContentPane(), BoxLayout.X_AXIS));
		this.addMouseWheelListener(this);

		this.addMapComponents(this.getContentPane());

		this.tabbedPane = new JTabbedPane();
		this.tabbedPane.setMinimumSize(new Dimension(311, Integer.MAX_VALUE));
		this.tabbedPane.setMaximumSize(new Dimension(311, Integer.MAX_VALUE));
		this.tabbedPane.setPreferredSize(new Dimension(311, Integer.MAX_VALUE));

		JPanel tilesetComponents = new JPanel();
		tilesetComponents.setLayout(new BorderLayout());
		this.addTilesetComponents(tilesetComponents);
		this.tabbedPane.addTab("Gehdaten", tilesetComponents);

		JPanel eventComponents = new JPanel();
		this.addEntityComponents(eventComponents);
		this.tabbedPane.addTab("Events", eventComponents);

		this.tabbedPane.addChangeListener(this);

		this.getContentPane().add(this.tabbedPane);
		this.pack();
	}
	
	private void addMapComponents(Container container) {
		this.imageMap = new GuiImageMap();

		Box imageBox = Box.createHorizontalBox();
		imageBox.add(Box.createHorizontalGlue());
		imageBox.add(this.imageMap);
		imageBox.add(Box.createHorizontalGlue());

		this.mapScrollPane = new JScrollPane(imageBox);
		this.mapScrollPane.removeMouseWheelListener(this.mapScrollPane.getMouseWheelListeners()[0]);
		this.mapScrollPane.addMouseWheelListener(this);
		container.add(this.mapScrollPane);
	}
	
	private void addTilesetComponents(Container container) {
		this.walkDataList = new GuiWalkDataList();
		this.tilesetList = new GuiTilesetList();

		JScrollPane walkDataScrollPane = new JScrollPane(this.walkDataList);
		walkDataScrollPane.setPreferredSize(new Dimension(307, 64));
		walkDataScrollPane.setMaximumSize(new Dimension(307, 64));
		walkDataScrollPane.setMinimumSize(new Dimension(307, 64));

		JScrollPane tilesetScrollPane = new JScrollPane(this.tilesetList, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		tilesetScrollPane.setPreferredSize(new Dimension(307, 400));
		tilesetScrollPane.setMaximumSize(new Dimension(307, 400));
		tilesetScrollPane.setMinimumSize(new Dimension(307, 400));

		container.add(walkDataScrollPane, BorderLayout.NORTH);
		container.add(tilesetScrollPane, BorderLayout.SOUTH);
	}

	private void addEntityComponents(Container container) {
		this.selectedEventTypeLabel = new JLabel("Nichts");
		this.selectedEventTypeLabel.setPreferredSize(new Dimension(90, 15));
		this.selectedEventTypeLabel.setMaximumSize(new Dimension(90, 15));
		this.selectedEventTypeLabel.setMinimumSize(new Dimension(90, 15));

		this.eventTypesList = new GuiEventTypesList();
		this.eventTypesList.setPreferredSize(new Dimension(72, 18));
		this.eventTypesList.setMaximumSize(new Dimension(72, 18));
		this.eventTypesList.setMinimumSize(new Dimension(72, 18));

		Box entityTypesBox = Box.createHorizontalBox();
		entityTypesBox.add(this.selectedEventTypeLabel);
		entityTypesBox.add(Box.createHorizontalStrut(10));
		entityTypesBox.add(this.eventTypesList);

		this.eventBox = Box.createVerticalBox();
		this.eventBox.add(entityTypesBox);

		container.add(this.eventBox);
	}
	
	public void showGui() {
		this.setVisible(true);
	}
	
	public void updateTileset() {
		this.tilesetList.reload();
		this.imageMap.reload();
	}

	public MapWalkData.WalkDataType getSelectedWalkData() {
		int index = this.walkDataList.getSelectedIndex();
		return MapWalkData.WalkDataType.getFromId(index);
	}

	public EventType getSelectedEventType() {
		int index = this.eventTypesList.getSelectedIndex();
		return EventType.getFromId(index);
	}

	public void updateSelectedEventTypeName() {
		EventType type = this.getSelectedEventType();
		this.selectedEventTypeLabel.setText(type == null ? "Nichts" : type.getName());
	}

	public int getSelectedTabID() {
		if (this.tabbedPane == null) {
			return -1;
		}
		return this.tabbedPane.getSelectedIndex();
	}

	public GuiImageMap getImageMap() {
		return this.imageMap;
	}

	public void saveSelectedEvent() {
		Event selectedEvent = this.imageMap.getSelectedEvent();
		if (selectedEvent != null) {
			selectedEvent.readFromGuiComponents(this.eventBox);
		}
	}

	@Override
	public void mouseWheelMoved(MouseWheelEvent event) {
		if (event.isControlDown()) {
			if (event.getWheelRotation() < 0) {
				// Wheel up
				this.imageMap.doZoom(1);
			} else {
				// Wheel down
				this.imageMap.doZoom(-1);
			}
		} else if (event.isShiftDown()) {
			JScrollBar bar = this.mapScrollPane.getHorizontalScrollBar();
			int newValue = Math.max(bar.getMinimum(), Math.min(bar.getMaximum(), bar.getValue() + event.getWheelRotation() * 10));
			bar.setValue(newValue);
		} else {
			JScrollBar bar = this.mapScrollPane.getVerticalScrollBar();
			int newValue = Math.max(bar.getMinimum(), Math.min(bar.getMaximum(), bar.getValue() + event.getWheelRotation() * 10));
			bar.setValue(newValue);
		}
	}

	public void selectEvent(Event event) {
		this.saveSelectedEvent();

		Component firstComponent = this.eventBox.getComponent(0);
		this.eventBox.removeAll();
		this.eventBox.add(firstComponent);

		if (event != null) {
			this.eventBox.add(Box.createVerticalStrut(6));
			event.addGuiComponents(this.eventBox);
			this.eventBox.add(Box.createVerticalStrut(1000));
		}

		this.tabbedPane.updateUI();
	}

	@Override
	public void stateChanged(ChangeEvent event) {
		if (this.imageMap.getCurrentZoomThread() != null) {
			try {
				this.imageMap.getCurrentZoomThread().join();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		this.imageMap.selectEvent(null);
		this.imageMap.doZoom(0);
	}
	
}
