package de.howaner.MapEditor.gui;

import de.howaner.MapEditor.MainMapEditor;
import de.howaner.MapEditor.gui.components.GuiImageMap;
import de.howaner.MapEditor.map.tileset.TileAnimation;
import java.awt.BorderLayout;
import java.awt.FileDialog;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FilenameFilter;
import java.util.logging.Level;
import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

public class TilesetPropertiesDialog extends JDialog implements ActionListener {
	private final MyWindow window;
	private final short tile;
	private JLabel tileImage;

	//Animation Type
	private JComboBox animationTypeChooser;

	//Animation Image
	private BufferedImage animationImage;
	private JLabel animationImageLabel;
	private JButton imageLoadButton;

	//Animation Duration
	private JTextField animationDurationField;

	//Animation Delay
	private JTextField animationDelayField;

	//Particle Name
	private JTextField particleNameField;

	//Particle Duration
	private JTextField particleDurationField;

	//Hide Player Foot
	private JCheckBox hidePlayerFootCheckBox;

	//Buttons
	private JButton cancelButton;
	private JButton saveButton;

	public TilesetPropertiesDialog(MyWindow window, short tile) {
		super(window, "Tileset bearbeiten: Tile #" + tile);
		this.window = window;
		this.tile = tile;
		this.getContentPane().setLayout(new BorderLayout());
		this.setSize(500, 345);
		this.setResizable(false);

		this.addComponents();
		this.loadProperties();
	}

	private void addComponents() {
		BufferedImage image = MainMapEditor.instance.getLoadedMap().getTileset().getTileFromId(this.tile);
		this.tileImage = new JLabel(new ImageIcon(GuiImageMap.zoom(image, 4)));
		this.tileImage.setSize(64, 64);

		Box imageBox = Box.createHorizontalBox();
		imageBox.add(Box.createHorizontalGlue());
		imageBox.add(this.tileImage);
		imageBox.add(Box.createHorizontalGlue());

		//---- Animation Box ----//

		// Animation Type
		this.animationTypeChooser = new JComboBox(new String[] { "Ohne Animation", "Normal", "Gehen", "Stehen" });
		this.animationTypeChooser.setSelectedIndex(0);

		Box animationTypeBox = Box.createHorizontalBox();
		animationTypeBox.add(new JLabel("Animation Typ: "));
		animationTypeBox.add(this.animationTypeChooser);

		// Animation Image
		this.animationImageLabel = new JLabel();
		this.updateAnimationImageLabel();

		this.imageLoadButton = new JButton("Laden");
		this.imageLoadButton.setActionCommand("Laden");
		this.imageLoadButton.addActionListener(this);

		Box animationImageBox = Box.createHorizontalBox();
		animationImageBox.add(new JLabel("Animation Image: "));
		animationImageBox.add(this.animationImageLabel);
		animationImageBox.add(Box.createHorizontalGlue());
		animationImageBox.add(this.imageLoadButton);

		// Animation Duration
		this.animationDurationField = new JTextField("0.10");

		Box animationDurationBox = Box.createHorizontalBox();
		animationDurationBox.add(new JLabel("Animation Dauer: "));
		animationDurationBox.add(Box.createHorizontalGlue());
		animationDurationBox.add(this.animationDurationField);

		// Animation Delay
		this.animationDelayField = new JTextField("1.0");

		Box animationDelayBox = Box.createHorizontalBox();
		animationDelayBox.add(new JLabel("Animation Verzögerung: "));
		animationDelayBox.add(Box.createHorizontalGlue());
		animationDelayBox.add(this.animationDelayField);

		// Animation Panel
		JPanel animationPanel = new JPanel();
		animationPanel.setLayout(new BoxLayout(animationPanel, BoxLayout.Y_AXIS));
		animationPanel.setBorder(BorderFactory.createTitledBorder("Animation"));
		animationPanel.add(animationTypeBox);
		animationPanel.add(animationImageBox);
		animationPanel.add(animationDurationBox);
		animationPanel.add(animationDelayBox);

		//---- Particle Box ----//

		// Particle Name
		this.particleNameField = new JTextField();

		Box particleNameBox = Box.createHorizontalBox();
		particleNameBox.add(new JLabel("Partikel Name: "));
		particleNameBox.add(Box.createHorizontalGlue());
		particleNameBox.add(this.particleNameField);

		// Particle Duration
		this.particleDurationField = new JTextField("0.25");

		Box particleDurationBox = Box.createHorizontalBox();
		particleDurationBox.add(new JLabel("Partikel Dauer: "));
		particleDurationBox.add(Box.createHorizontalGlue());
		particleDurationBox.add(this.particleDurationField);

		// Particle Panel
		JPanel particlePanel = new JPanel();
		particlePanel.setLayout(new BoxLayout(particlePanel, BoxLayout.Y_AXIS));
		particlePanel.setBorder(BorderFactory.createTitledBorder("Partikel"));
		particlePanel.add(particleNameBox);
		particlePanel.add(particleDurationBox);

		// Hide Player Foot Checkbox
		this.hidePlayerFootCheckBox = new JCheckBox("Spielerfuß verstecken");

		//---- Buttons ----//
		this.cancelButton = new JButton("Abbrechen");
		this.cancelButton.setActionCommand("Abbrechen");
		this.cancelButton.addActionListener(this);
		this.saveButton = new JButton("Speichern");
		this.saveButton.setActionCommand("Speichern");
		this.saveButton.addActionListener(this);

		// Button Box
		Box buttonBox = Box.createHorizontalBox();
		buttonBox.add(Box.createHorizontalGlue());
		buttonBox.add(this.cancelButton);
		buttonBox.add(Box.createHorizontalStrut(3));
		buttonBox.add(this.saveButton);
		buttonBox.add(Box.createHorizontalStrut(1));

		//------------------------//
		Box content = Box.createVerticalBox();
		content.add(animationPanel);
		content.add(Box.createVerticalStrut(10));
		content.add(particlePanel);
		content.add(Box.createVerticalStrut(4));
		content.add(this.hidePlayerFootCheckBox);
		content.add(Box.createVerticalStrut(10));
		content.add(buttonBox);
		content.add(Box.createVerticalStrut(1));

		this.getContentPane().add(this.tileImage, BorderLayout.CENTER);
		this.getContentPane().add(content, BorderLayout.PAGE_END);
	}

	public void updateAnimationImageLabel() {
		if (this.animationImage == null) {
			return;
		}

		BufferedImage scaledImage = scaleImage(this.animationImage, 272, 16);
		this.animationImageLabel.setIcon(new ImageIcon(scaledImage));
	}

	public static BufferedImage scaleImage(BufferedImage image, int maxWidth, int maxHeight) {
		int newWidth = image.getWidth();
		int newHeight = image.getHeight();

		float scale = (float)image.getHeight() / (float)image.getWidth();
		if (newHeight > maxHeight) {
			newHeight = maxHeight;
			newWidth = (int) (newHeight * scale);
		}

		scale = (float)image.getHeight() / (float)image.getWidth(); //Update scale
		if (newWidth > maxWidth) {
			newWidth = maxWidth;
			newHeight = (int) (newWidth * scale);
		}

		BufferedImage resizedImage = new BufferedImage(maxWidth, maxHeight, BufferedImage.TYPE_INT_ARGB);
		Graphics2D g = resizedImage.createGraphics();
		g.drawImage(image, 0, 0, newWidth, newHeight, null);
		g.dispose();

		return resizedImage;
	}

	@Override
	public void actionPerformed(ActionEvent event) {
		if (event.getActionCommand().equals("Laden")) {
			FileDialog dialog = new FileDialog(MainMapEditor.instance.getWindow(), "Wähle ein Bild aus", FileDialog.LOAD);
			dialog.setFilenameFilter(new FilenameFilter() {
				@Override
				public boolean accept(File file, String fileName) {
					return fileName.endsWith(".png");
				}
			});
			dialog.setVisible(true);

			if (dialog.getFile() != null) {
				try {
					this.animationImage = ImageIO.read(dialog.getFiles()[0]);
					this.updateAnimationImageLabel();
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(MainMapEditor.instance.getWindow(), "Can't load picture: " + ex.getMessage(), "Can't load picture.", JOptionPane.ERROR_MESSAGE);
					MainMapEditor.logger.log(Level.WARNING, "Can't load picture", ex);
				}
			}
		} else if (event.getActionCommand().equals("Abbrechen")) {
			this.setVisible(false);
		} else if (event.getActionCommand().equals("Speichern")) {
			this.saveAndClose();
		}
	}

	public void loadProperties() {
		TileAnimation animation = MainMapEditor.instance.getLoadedMap().getTileset().getTileAnimation(this.tile);
		if (animation == null) {
			return;
		}

		if (animation.getAnimationType() != null) {
			this.animationTypeChooser.setSelectedIndex(animation.getAnimationType().ordinal() + 1);
			this.animationImage = animation.getAnimationImage();
			this.animationDurationField.setText(String.valueOf(animation.getAnimationDuration()));
			this.animationDelayField.setText(String.valueOf(animation.getAnimationDelay()));
			this.updateAnimationImageLabel();
		}

		if (!animation.getParticleName().isEmpty()) {
			this.particleNameField.setText(animation.getParticleName());
			this.particleDurationField.setText(String.valueOf(animation.getParticleDuration()));
		}

		this.hidePlayerFootCheckBox.setSelected(animation.getHidePlayerFoot());
	}

	public void saveAndClose() {
		TileAnimation animation = new TileAnimation();

		if (this.animationTypeChooser.getSelectedIndex() != 0) {
			animation.setAnimationType(TileAnimation.TileAnimationType.values()[this.animationTypeChooser.getSelectedIndex() - 1]);

			if (this.animationImage == null) {
				JOptionPane.showMessageDialog(this.window, "Bitte lade ein Animation Bild!", "Fehler", JOptionPane.ERROR_MESSAGE);
				return;
			}
			animation.setAnimationImage(this.animationImage);

			if (this.animationDurationField.getText().isEmpty()) {
				JOptionPane.showMessageDialog(this.window, "Keine Animation Dauer angegeben!", "Fehler", JOptionPane.ERROR_MESSAGE);
				return;
			}
			float animationDuration;
			try {
				animationDuration = Float.parseFloat(this.animationDurationField.getText());
			} catch (Exception ex) {
				JOptionPane.showMessageDialog(this.window, "Bitte geben sie eine richtige Animation Dauer Zahl an!", "Fehler", JOptionPane.ERROR_MESSAGE);
				return;
			}
			animation.setAnimationDuration(animationDuration);

			if (this.animationDelayField.getText().isEmpty()) {
				JOptionPane.showMessageDialog(this.window, "Keine Animation Verzögerung angegeben!", "Fehler", JOptionPane.ERROR_MESSAGE);
				return;
			}
			float animationDelay;
			try {
				animationDelay = Float.parseFloat(this.animationDelayField.getText());
			} catch (Exception ex) {
				JOptionPane.showMessageDialog(this.window, "Bitte geben sie eine richtige Animation Verzögerung Zahl an!", "Fehler", JOptionPane.ERROR_MESSAGE);
				return;
			}
			animation.setAnimationDelay(animationDelay);
		}

		if (!this.particleNameField.getText().isEmpty()) {
			animation.setParticleName(this.particleNameField.getText());

			if (this.particleDurationField.getText().isEmpty()) {
				JOptionPane.showMessageDialog(this.window, "Keine Particle Dauer angegeben!", "Fehler", JOptionPane.ERROR_MESSAGE);
				return;
			}
			float particleDuration;
			try {
				particleDuration = Float.parseFloat(this.particleDurationField.getText());
			} catch (Exception ex) {
				JOptionPane.showMessageDialog(this.window, "Bitte geben sie eine richtige Particle Dauer an!", "Fehler", JOptionPane.ERROR_MESSAGE);
				return;
			}
			animation.setParticleDuration(particleDuration);
		}

		animation.setHidePlayerFoot(this.hidePlayerFootCheckBox.isSelected());

		if (animation.isEmpty()) {
			animation = null;
		}

		MainMapEditor.instance.getLoadedMap().getTileset().setTileAnimation(this.tile, animation);
		this.setVisible(false);
	}

}
