package de.howaner.MapEditor.gui.components;

import de.howaner.MapEditor.MainMapEditor;
import de.howaner.MapEditor.gui.MapPropertiesDialog;
import de.howaner.MapEditor.map.MapConverter;
import de.howaner.MapEditor.map.MapImageSerializer;
import de.howaner.MapEditor.map.MapTerrain;
import de.howaner.MapEditor.map.events.Event;
import de.howaner.MapEditor.map.tileset.MapTileset;
import de.howaner.MapEditor.map.tileset.TilesetConverter;
import de.howaner.Pokemon.saveformat.TagCompound;
import de.howaner.Pokemon.saveformat.TagManager;
import java.awt.FileDialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FilenameFilter;
import java.util.logging.Level;
import javax.imageio.ImageIO;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

public class GuiMenuBar extends JMenuBar implements ActionListener {

	// --- Menues --- //
	private JMenu fileMenu;
	private JMenu mapMenu;
	private JMenu tilesetMenu;

	public GuiMenuBar() {

		// Add "File Menu"
		this.fileMenu = new JMenu("Datei");
		this.fileMenu.add(createMenuItem("Neu"));
		this.fileMenu.add(createMenuItem("Laden"));
		this.fileMenu.addSeparator();
		this.fileMenu.add(createMenuItem("Speichern"));
		this.fileMenu.add(createMenuItem("Speichern unter"));

		// Add "Map Menu"
		this.mapMenu = new JMenu("Welt");
		this.mapMenu.add(createMenuItem("Importiere Map"));
		this.mapMenu.add(createMenuItem("Speichere Mapbild"));
		this.mapMenu.add(createMenuItem("Map Eigenschaften"));

		// Add "Tileset Menu"
		this.tilesetMenu = new JMenu("Tileset");
		this.tilesetMenu.add(createMenuItem("Lade Tileset"));
		this.tilesetMenu.add(createMenuItem("Lade Tileset Bild"));
		this.tilesetMenu.addSeparator();
		this.tilesetMenu.add(createMenuItem("Speichere Tileset"));
		this.tilesetMenu.add(createMenuItem("Speichere Tileset Bild"));

		this.add(fileMenu);
		this.add(mapMenu);
		this.add(tilesetMenu);
	}

	@Override
	public void actionPerformed(ActionEvent event) {
		if (event.getActionCommand().equals("Neu")) {
			MainMapEditor.instance.getWindow().getImageMap().selectEvent(null);
			MainMapEditor.instance.setMap(new MapTerrain());
			MainMapEditor.instance.getWindow().updateTileset();
		}
		else if (event.getActionCommand().equals("Laden")) {
			FileDialog dialog = new FileDialog(MainMapEditor.instance.getWindow(), "Wähle eine Map aus", FileDialog.LOAD);
			dialog.setFilenameFilter(new FilenameFilter() {
				@Override
				public boolean accept(File file, String fileName) {
					return fileName.endsWith(".pkmap");
				}
			});
			dialog.setVisible(true);

			if (dialog.getFile() != null) {
				try {
					TagCompound tag = TagManager.readFromFile(dialog.getFiles()[0]);

					MapTerrain map = new MapTerrain();
					map.loadFromCompound(tag);

					String name = dialog.getFiles()[0].getName();
					map.setName(name.substring(0, name.length() - ".pkmap".length()));

					MainMapEditor.instance.getWindow().getImageMap().selectEvent(null);
					MainMapEditor.instance.setMap(map);
					MainMapEditor.instance.getWindow().updateTileset();
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(MainMapEditor.instance.getWindow(), "Can't load map: " + ex.getMessage(), "Can't load map.", JOptionPane.ERROR_MESSAGE);
					MainMapEditor.logger.log(Level.WARNING, "Can't load map", ex);
				}
			}
		} else if (event.getActionCommand().equals("Speichern")) {
			FileDialog dialog = new FileDialog(MainMapEditor.instance.getWindow(), "Wähle einen Speicherort für die Map aus", FileDialog.SAVE);
			dialog.setFile("*.pkmap");
			dialog.setVisible(true);

			if (dialog.getFile() != null) {
				try {
					TagCompound tag = new TagCompound();
					MainMapEditor.instance.getWindow().saveSelectedEvent();
					MainMapEditor.instance.getLoadedMap().saveToCompound(tag);

					TagManager.writeToFile(tag, dialog.getFiles()[0]);

					String name = dialog.getFiles()[0].getName();
					MainMapEditor.instance.getLoadedMap().setName(name.substring(0, name.length() - ".pkmap".length()));
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(MainMapEditor.instance.getWindow(), "Can't save map: " + ex.getMessage(), "Can't save map.", JOptionPane.ERROR_MESSAGE);
					MainMapEditor.logger.log(Level.WARNING, "Can't save map", ex);
				}
			}
		} else if (event.getActionCommand().equals("Importiere Map")) {
			FileDialog dialog = new FileDialog(MainMapEditor.instance.getWindow(), "Wähle eine Map zum importieren aus", FileDialog.LOAD);
			dialog.setFilenameFilter(new FilenameFilter() {
				@Override
				public boolean accept(File file, String fileName) {
					return fileName.endsWith(".tmx");
				}
			});
			dialog.setVisible(true);

			if (dialog.getFile() != null) {
				// Load the tileset
				try {
					MainMapEditor.instance.getWindow().saveSelectedEvent();

					MapTerrain oldMap = MainMapEditor.instance.getLoadedMap();
					MapConverter mapConverter = new MapConverter();
					MapTerrain map = mapConverter.convertMap(dialog.getFiles()[0]);

					map.setName(oldMap.getName());
					for (int x = 0; x < Math.min(map.getWidth(), oldMap.getWidth()); x++) {
						for (int y = 0; y < Math.min(map.getHeight(), oldMap.getHeight()); y++) {
							map.getWalkData().setWalkData(x, y, oldMap.getWalkData().getWalkData(x, y));
						}
					}

					for (int x = 0; x < Math.min(map.getWidth(), oldMap.getWidth()); x++) {
						for (int y = 0; y < Math.min(map.getHeight(), oldMap.getHeight()); y++) {
							map.getEvents().setEvent(x, y, oldMap.getEvents().getEventAt(x, y));
						}
					}

					MainMapEditor.instance.setMap(map);
					MainMapEditor.instance.getWindow().updateTileset();
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(MainMapEditor.instance.getWindow(), "Can't load map tiles: " + ex.getMessage(), "Can't load map tiles.", JOptionPane.ERROR_MESSAGE);
					MainMapEditor.logger.log(Level.WARNING, "Can't load map", ex);
				}
			}
		} else if (event.getActionCommand().equals("Speichere Mapbild")) {
			FileDialog dialog = new FileDialog(MainMapEditor.instance.getWindow(), "Wähle einen Speicherort für das Bild aus", FileDialog.SAVE);
			dialog.setFile("*.png");

			dialog.setVisible(true);
			if (dialog.getFile() != null) {
				MapTerrain map = MainMapEditor.instance.getLoadedMap();
				BufferedImage mapImage = MapImageSerializer.serializeMapToImage(map);
				try {
					ImageIO.write(mapImage, "png", dialog.getFiles()[0]);
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(MainMapEditor.instance.getWindow(), "Can't save map picture: " + ex.getMessage(), "Can't save map picture.", JOptionPane.ERROR_MESSAGE);
					MainMapEditor.logger.log(Level.WARNING, "Can't save map picture.", ex);
				}
			}
		} else if (event.getActionCommand().equals("Lade Tileset")) {
			FileDialog dialog = new FileDialog(MainMapEditor.instance.getWindow(), "Wähle ein Tileset aus", FileDialog.LOAD);
			dialog.setFilenameFilter(new FilenameFilter() {
				@Override
				public boolean accept(File file, String fileName) {
					return fileName.endsWith(".pktileset");
				}
			});
			dialog.setVisible(true);

			if (dialog.getFile() != null) {
				try {
					TagCompound compound = TagManager.readFromFile(dialog.getFiles()[0]);

					MapTileset tileset = new MapTileset();
					tileset.loadFromCompound(compound);

					MainMapEditor.instance.getLoadedMap().setTileset(tileset);
					MainMapEditor.instance.getWindow().updateTileset();
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(MainMapEditor.instance.getWindow(), "Can't load tileset: " + ex.getMessage(), "Can't load tileset.", JOptionPane.ERROR_MESSAGE);
					MainMapEditor.logger.log(Level.WARNING, "Can't load tileset", ex);
				}
			}
		} else if (event.getActionCommand().equals("Lade Tileset Bild")) {
			FileDialog dialog = new FileDialog(MainMapEditor.instance.getWindow(), "Wähle ein Tileset Bild aus", FileDialog.LOAD);
			dialog.setFilenameFilter(new FilenameFilter() {
				@Override
				public boolean accept(File file, String fileName) {
					return fileName.endsWith(".png");
				}
			});
			dialog.setVisible(true);

			if (dialog.getFile() != null) {
				try {
					MapTileset tileset = TilesetConverter.convertFileToTileset(dialog.getFiles()[0]);
					MainMapEditor.instance.getLoadedMap().getTileset().setTiles(tileset.getTiles());
					MainMapEditor.instance.getLoadedMap().getTileset().setTilesPerLine(tileset.getTilesPerLine());
					MainMapEditor.instance.getWindow().updateTileset();
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(MainMapEditor.instance.getWindow(), "Can't load tileset image: " + ex.getMessage(), "Can't load tileset image.", JOptionPane.ERROR_MESSAGE);
					MainMapEditor.logger.log(Level.WARNING, "Can't load tileset image", ex);
				}
			}
		} else if (event.getActionCommand().equals("Speichere Tileset")) {
			FileDialog dialog = new FileDialog(MainMapEditor.instance.getWindow(), "Wählen sie einen Speicherort aus", FileDialog.SAVE);
			dialog.setFile("*.pktileset");
			dialog.setVisible(true);

			if (dialog.getFile() != null) {
				// Save tileset
				try {
					MapTileset tileset = MainMapEditor.instance.getLoadedMap().getTileset();

					TagCompound compound = new TagCompound();
					tileset.saveToCompound(compound);

					TagManager.writeToFile(compound, dialog.getFiles()[0]);
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(MainMapEditor.instance.getWindow(), "Can't save tileset: " + ex.getMessage(), "Can't save tileset.", JOptionPane.ERROR_MESSAGE);
					MainMapEditor.logger.log(Level.WARNING, "Can't save tileset", ex);
				}
			}
		} else if (event.getActionCommand().equals("Speichere Tileset Bild")) {
			FileDialog dialog = new FileDialog(MainMapEditor.instance.getWindow(), "Wählen sie einen Speicherort aus", FileDialog.SAVE);
			dialog.setFile("*.png");
			dialog.setVisible(true);

			if (dialog.getFile() != null) {
				// Save tileset image
				try {
					MapTileset tileset = MainMapEditor.instance.getLoadedMap().getTileset();
					TilesetConverter.writeTilesetToFile(tileset, dialog.getFiles()[0]);
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(MainMapEditor.instance.getWindow(), "Can't save tileset image: " + ex.getMessage(), "Can't save tileset image.", JOptionPane.ERROR_MESSAGE);
					MainMapEditor.logger.log(Level.WARNING, "Can't save tileset image", ex);
				}
			}
		} else if (event.getActionCommand().equals("Map Eigenschaften")) {
			new MapPropertiesDialog(MainMapEditor.instance.getWindow()).setVisible(true);
		}
	}

	private JMenuItem createMenuItem(String name) {
		JMenuItem menuItem = new JMenuItem(name);
		menuItem.addActionListener(this);
		menuItem.setActionCommand(name);
		return menuItem;
	}
}
