package de.howaner.MapEditor.gui.components;

import de.howaner.MapEditor.MainMapEditor;
import de.howaner.MapEditor.gui.TilesetPropertiesDialog;
import java.awt.Component;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListSelectionModel;

public class GuiTilesetList extends JList implements MouseListener {

	public GuiTilesetList() {
		super();

		this.addMouseListener(this);
		this.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		this.setLayoutOrientation(JList.HORIZONTAL_WRAP);
		this.setVisibleRowCount(-1);
		this.setCellRenderer(new TilesetIconRenderer());

		this.reload();
	}

	public void reload() {
		DefaultListModel model = new DefaultListModel();
		for (int i = 0; i < MainMapEditor.instance.getLoadedMap().getTileset().getTiles().size(); i++) {
			model.addElement("");
		}

		this.setModel(model);
		this.setVisibleRowCount(MainMapEditor.instance.getLoadedMap().getTileset().getTilesPerLine());

		if (this.getVisibleRowCount() == 16)
			this.setVisibleRowCount(-1);
	}

	public class TilesetIconRenderer extends DefaultListCellRenderer {
		@Override
		public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
			BufferedImage image = MainMapEditor.instance.getLoadedMap().getTileset().getTileFromId(index);
			Icon icon = new ImageIcon(image);

			JLabel label = (JLabel) super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			label.setIcon(icon);
			return label;
		}
	}

	@Override
	public void mouseClicked(MouseEvent event) {
		if (event.getClickCount() == 2) {
			// Double click
			if (this.getSelectedIndex() < 0 || this.getSelectedIndex() >= MainMapEditor.instance.getLoadedMap().getTileset().getTiles().size()) {
				return;
			}

			TilesetPropertiesDialog dialog = new TilesetPropertiesDialog(MainMapEditor.instance.getWindow(), (short) this.getSelectedIndex());
			dialog.setVisible(true);
		}
	}

	@Override
	public void mousePressed(MouseEvent event) {
		
	}

	@Override
	public void mouseReleased(MouseEvent event) {
		
	}

	@Override
	public void mouseEntered(MouseEvent event) {
		
	}

	@Override
	public void mouseExited(MouseEvent event) {
		
	}

}
