package de.howaner.MapEditor.map;

import de.howaner.MapEditor.MainMapEditor;
import java.io.File;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class MapConverter {
	private Document doc;

	public MapTerrain convertMap(File file) throws Exception {
		DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
		this.doc = dBuilder.parse(file);

		Element mapElement = this.doc.getDocumentElement();
		mapElement.normalize();
		if (!mapElement.getAttribute("orientation").equals("orthogonal")) throw new Exception("Only orthogonal maps are supported!");
		int width = Integer.parseInt(mapElement.getAttribute("width"));
		int height = Integer.parseInt(mapElement.getAttribute("height"));

		MapTerrain map = new MapTerrain("Map " + file.getName(), width, height);
		this.loadLayers(map, mapElement);
		map.setTileset(MainMapEditor.instance.getLoadedMap().getTileset());
		return map;
	}

	public void loadLayers(MapTerrain map, Element mapElement) {
		NodeList layers = mapElement.getElementsByTagName("layer");
		for (int i = 0; i < layers.getLength(); i++) {
			Node node = layers.item(i);
			if (node.getNodeType() != Node.ELEMENT_NODE) continue;

			Element layerElement = (Element)node;
			MapLayer layer = this.loadLayer(map, layerElement);

			if (layer != null) {
				map.addLayer(layer);
			}
		}
	}

	public MapLayer loadLayer(MapTerrain map, Element layerElement) {
		NodeList dataNodes = layerElement.getElementsByTagName("data");
		if (dataNodes.getLength() == 0) {
			return null;
		}

		MapLayer layer = new MapLayer(map.getLayers().size(), map);

		Node dataNode = dataNodes.item(0);
		if (dataNode.getNodeType() != Node.ELEMENT_NODE) {
			return null;
		}
		Element dataElement = (Element)dataNode;

		int currentY = map.getHeight() - 1;
		int currentX = 0;
		NodeList tileNodes = dataElement.getElementsByTagName("tile");
		for (int i = 0; i < tileNodes.getLength(); i++) {
			Node tileNode = tileNodes.item(i);
			if (tileNode.getNodeType() != Node.ELEMENT_NODE) continue;
			Element tileElement = (Element)tileNode;
			short tile = Short.parseShort(tileElement.getAttribute("gid"));

			layer.setTile(currentX, currentY, tile);

			currentX++;
			if (currentX >= map.getWidth()) {
				currentY--;
				currentX = 0;
			}
		}

		return layer;
	}

}


