package de.howaner.MapEditor.map;

import de.howaner.MapEditor.map.tileset.MapTileset;
import java.awt.image.BufferedImage;

public class MapImageSerializer {

	public static BufferedImage serializeMapToImage(MapTerrain map) {
		System.out.println("Started map rendering!");
		if (map.getWidth() == 0 || map.getHeight() == 0) {
			return new BufferedImage(1, 1, BufferedImage.TYPE_INT_ARGB);
		}
		BufferedImage image = new BufferedImage(map.getWidth() * 16, map.getHeight() * 16, BufferedImage.TYPE_INT_ARGB);

		MapTileset tileset = map.getTileset();
		for (MapLayer layer : map.getLayers()) {
			for (int x = 0; x < map.getWidth(); x++) {
				for (int y = 0; y < map.getHeight(); y++) {
					int tileID = layer.getTile(x, y);
					if (tileID == 0) continue;
					BufferedImage tile = tileset.getTileFromId(tileID - 1);
					if (tile == null) {
						System.out.println("Invalid tile found! " + tileID);
						continue;
					}

					for (int tileX = 0; tileX < 16; tileX++) {
						for (int tileY = 0; tileY < 16; tileY++) {
							int rgb = tile.getRGB(tileX, tileY);
							int alpha = (rgb >> 24) & 0xff;
							if (alpha != 0)
								image.setRGB(x * 16 + tileX, (map.getHeight() - 1 - y) * 16 + tileY, rgb);
						}
					}
				}
			}
		}

		System.out.println("Map rendering finished!");
		return image;
	}

}
