package de.howaner.MapEditor.map;

import de.howaner.MapEditor.MainMapEditor;
import de.howaner.Pokemon.saveformat.TagBase;
import de.howaner.Pokemon.saveformat.TagShort;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;

public class MapLayer {
	private int id;
	private final MapTerrain map;
	private short[] tiles;

	public MapLayer(int id, MapTerrain map) {
		this.id = id;
		this.map = map;
		this.tiles = new short[map.getWidth() * map.getHeight()];
	}

	public int getId() {
		return this.id;
	}

	public MapTerrain getMap() {
		return this.map;
	}

	public short[] getTiles() {
		return this.tiles;
	}

	public short getTile(int x, int y) {
		if (x >= this.map.getWidth() || y >= this.map.getHeight()) {
			MainMapEditor.logger.log(Level.WARNING, "getTile(): Too high x/y value. (X: {0}, Y: {1}, Max X: {2}, Max Y: {3})", new Object[]{x, y, this.map.getWidth(), this.map.getHeight()});
			return 0;
		}
		return this.tiles[y * this.map.getWidth() + x];
	}

	public void setTile(int x, int y, short tile) {
		if (x >= this.map.getWidth() || y >= this.map.getHeight()) {
			MainMapEditor.logger.log(Level.WARNING, "setTile(): Too high x/y value. (X: {0}, Y: {1}, Max X: {2}, Max Y: {3})", new Object[]{x, y, this.map.getWidth(), this.map.getHeight()});
			return;
		}
		this.tiles[y * this.map.getWidth() + x] = tile;
	}
	
	public void loadFromTagList(List<TagBase> list) {
		this.tiles = new short[map.getWidth() * map.getHeight()];
		for (int i = 0; i < list.size(); i++) {
			if (i >= this.tiles.length) {
				break;
			}

			TagShort tag = (TagShort) list.get(i);
			this.tiles[i] = tag.getData();
		}
	}
	
	public List<TagBase> getTagList() {
		List<TagBase> list = new ArrayList<TagBase>();
		for (short tile : this.tiles) {
			list.add(new TagShort(tile));
		}

		return list;
	}

}
