package de.howaner.MapEditor.map;

import de.howaner.MapEditor.map.events.MapEvents;
import de.howaner.MapEditor.map.tileset.MapTileset;
import de.howaner.Pokemon.saveformat.TagBase;
import de.howaner.Pokemon.saveformat.TagCompound;
import de.howaner.Pokemon.saveformat.TagList;
import java.util.ArrayList;
import java.util.List;

public class MapTerrain {
	private String name;
	private String joinMessage = "";
	private int width;
	private int height;
	private List<MapLayer> layers = new ArrayList<MapLayer>();
	private MapTileset tileset = new MapTileset();
	private MapWalkData walkData;
	private MapEvents events;

	public MapTerrain() {
		this("Unknown", 40, 40);
	}

	public MapTerrain(String name, int width, int height) {
		this.name = name;
		this.width = width;
		this.height = height;
		this.walkData = new MapWalkData(this);
		this.events = new MapEvents(this);
	}

	public MapWalkData getWalkData() {
		return this.walkData;
	}

	public MapEvents getEvents() {
		return this.events;
	}
	
	public void setName(String name) {
		this.name = name;
	}

	public String getName() {
		return this.name;
	}

	public String getJoinMessage() {
		return this.joinMessage;
	}

	public void setJoinMessage(String joinMessage) {
		this.joinMessage = joinMessage;
	}

	public int getWidth() {
		return this.width;
	}

	public int getHeight() {
		return this.height;
	}

	public void addLayer(MapLayer layer) {
		this.layers.add(layer);
	}

	public void removeLayer(MapLayer layer) {
		this.layers.remove(layer);
	}

	public MapLayer getLayer(int id) {
		return this.layers.get(id);
	}

	public List<MapLayer> getLayers() {
		return this.layers;
	}

	public MapTileset getTileset() {
		return this.tileset;
	}

	public void setTileset(MapTileset tileset) {
		this.tileset = tileset;
	}
	
	public void setWalkData(MapWalkData walkData) {
		this.walkData = walkData;
	}

	public void setEvents(MapEvents events) {
		this.events = events;
	}
	
	public void loadFromCompound(TagCompound compound) {
		if (compound.containsTag("joinMessage"))
			this.joinMessage = compound.getString("joinMessage");
		this.width = compound.getInt("width");
		this.height = compound.getInt("height");

		if (compound.containsTag("tileset")) {
			TagCompound tilesetCompound = compound.getCompound("tileset");
			this.tileset.loadFromCompound(tilesetCompound);
		}

		// Load layers
		List<TagBase> layersList = compound.getList("layers");
		for (TagBase layerTag : layersList) {
			MapLayer layer = new MapLayer(this.layers.size(), this);
			layer.loadFromTagList(((TagList) layerTag).getData());
			this.addLayer(layer);
		}

		// Load walkdata
		this.walkData = new MapWalkData(this);
		this.walkData.loadFromTagList(compound.getList("walkData"));

		// Load entities
		if (compound.containsTag("events")) {
			this.events = new MapEvents(this);
			this.events.loadFromTagList(compound.getList("events"));
		}
	}

	public void saveToCompound(TagCompound compound) {
		compound.setString("joinMessage", this.joinMessage);
		compound.setInt("width", this.width);
		compound.setInt("height", this.height);

		// Save tileset
		if (this.tileset != null) {
			TagCompound tilesetCompound = new TagCompound();
			this.tileset.saveToCompound(tilesetCompound);
			compound.setCompound("tileset", tilesetCompound);
		}

		// Save layers
		List<TagBase> layerList = new ArrayList<TagBase>();
		for (MapLayer layer : this.getLayers()) {
			layerList.add(new TagList(10, layer.getTagList()));
		}
		compound.setList("layers", 8, layerList);

		// Save walk data
		List<TagBase> walkDataList = new ArrayList<TagBase>();
		this.walkData.saveToTagList(walkDataList);
		compound.setList("walkData", 10, walkDataList);

		// Save entities
		List<TagBase> eventsList = new ArrayList<TagBase>();
		this.events.saveToTagList(eventsList);
		compound.setList("events", 1, eventsList);
	}

}
