package de.howaner.MapEditor.map;

import de.howaner.MapEditor.MainMapEditor;
import de.howaner.Pokemon.saveformat.TagBase;
import de.howaner.Pokemon.saveformat.TagShort;
import java.awt.image.BufferedImage;
import java.io.InputStream;
import java.util.List;
import java.util.logging.Level;
import javax.imageio.ImageIO;

public class MapWalkData {
	private static BufferedImage walkDatasImage;

	private final MapTerrain map;
	private WalkDataType[] walkData;

	static {
		InputStream inputStream = MainMapEditor.class.getResourceAsStream("/walkdata.png");
		try {
			walkDatasImage = ImageIO.read(inputStream);
		} catch (Exception e) {
			MainMapEditor.logger.log(Level.SEVERE, "Can't load walkdata.png", e);
		}
	}

	public MapWalkData(MapTerrain map) {
		this.map = map;
		this.walkData = new WalkDataType[this.map.getWidth() * this.map.getHeight()];
		this.fillWalkData();
	}

	private void fillWalkData() {
		for (int i = 0; i < this.walkData.length; i++) {
			this.walkData[i] = WalkDataType.NONE;
		}
	}
	
	public WalkDataType[] getDataArray() {
		return this.walkData;
	}
	
	public WalkDataType getWalkData(int x, int y) {
		return this.walkData[y * this.map.getWidth() + x];
	}

	public void setWalkData(int x, int y, WalkDataType walkData) {
		this.walkData[y * this.map.getWidth() + x] = walkData;
	}

	public void loadFromTagList(List<TagBase> list) {
		this.walkData = new WalkDataType[this.map.getWidth() * this.map.getHeight()];
		this.fillWalkData();
		for (int i = 0; i < list.size(); i++) {
			TagShort tag = (TagShort) list.get(i);
			this.walkData[i] = WalkDataType.getFromId(tag.getData());
		}
	}
	
	public void saveToTagList(List<TagBase> list) {
		for (WalkDataType walkData : this.walkData) {
			list.add(new TagShort((short) walkData.ordinal()));
		}
	}

	public enum WalkDataType {
		NONE, Layer0, Layer1, Layer2, Layer3, Layer4, ONLY_NORTH, ONLY_EAST, ONLY_SOUTH, ONLY_WEST, BLOCKED, SURFER, DIVE, KASCADE;

		private BufferedImage imageCache;

		public int getWidthCoord() {
			return ((this.ordinal() % 2) == 0) ? 0 : 16;
		}

		public int getHeightCoord() {
			return (this.ordinal() / 2) * 16;
		}

		public BufferedImage getImage() {
			if (this.imageCache == null) {
				this.imageCache = MapWalkData.walkDatasImage.getSubimage(this.getWidthCoord(), this.getHeightCoord(), 16, 16);
			}
			return this.imageCache;
		}

		public static WalkDataType getFromId(int id) {
			if (id < 0 || id >= values().length) {
				return null;
			}

			return values()[id];
		}
	}

}
