package de.howaner.MapEditor.map.events;

import de.howaner.MapEditor.map.MapTerrain;
import de.howaner.Pokemon.saveformat.TagCompound;
import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JToggleButton;

public class EventInteract extends Event {
	private boolean requireInteract = false;
	private String scriptName = "";

	public EventInteract(MapTerrain map, int posX, int posY) {
		super(map, posX, posY);
	}

	public boolean isRequireInteract() {
		return this.requireInteract;
	}

	public void setRequireInteract(boolean requireInteract) {
		this.requireInteract = requireInteract;
	}

	public String getScriptName() {
		return this.scriptName;
	}

	public void setScriptName(String scriptName) {
		this.scriptName = scriptName;
	}

	@Override
	public void loadFromCompound(TagCompound compound) {
		super.loadFromCompound(compound);
		this.requireInteract = compound.getBoolean("RequireInteract");
		this.scriptName = compound.getString("ScriptName");
	}

	@Override
	public void saveToCompound(TagCompound compound) {
		super.saveToCompound(compound);
		compound.setBoolean("RequireInteract", this.requireInteract);
		compound.setString("ScriptName", this.scriptName);
	}

	@Override
	public void addGuiComponents(Box box) {
		JToggleButton toggleButton = new JToggleButton("Benötigt Interact", this.requireInteract);
		toggleButton.setToolTipText("Wenn das aktiviert ist, muss man die Interact Taste drücken, um das Script auszuführen. Wenn das deaktiviert ist, läuft man einfach dagegen, um das Script auszuführen.");

		JTextField scriptNameField = new JTextField();
		scriptNameField.setText(this.scriptName);

		Box scriptNameBox = Box.createHorizontalBox();
		scriptNameBox.add(new JLabel("Skript Name: "));
		scriptNameBox.add(Box.createHorizontalGlue());
		scriptNameBox.add(scriptNameField);

		box.add(toggleButton);
		box.add(scriptNameBox);
	}

	@Override
	public void readFromGuiComponents(Box box) {
		JToggleButton toggleButton = (JToggleButton) box.getComponent(box.getComponentCount() - 3);
		this.requireInteract = toggleButton.isSelected();

		Box scriptNameBox = (Box) box.getComponent(box.getComponentCount() - 2);
		JTextField scriptNameField = (JTextField) scriptNameBox.getComponent(2);

		this.scriptName = scriptNameField.getText();
	}

	@Override
	public EventType getType() {
		return EventType.INTERACT;
	}

}
