package de.howaner.MapEditor.map.events;

import de.howaner.MapEditor.MainMapEditor;
import de.howaner.MapEditor.map.MapTerrain;
import java.awt.image.BufferedImage;
import java.io.InputStream;
import java.util.logging.Level;
import javax.imageio.ImageIO;

public enum EventType {
	ENTITY, WARP, SCRIPT, INTERACT;

	private static BufferedImage eventTypesImage;
	private BufferedImage imageCache, selectedImageCache;

	static {
		InputStream inputStream = MainMapEditor.class.getResourceAsStream("/event_types.png");
		try {
			eventTypesImage = ImageIO.read(inputStream);
		} catch (Exception e) {
			MainMapEditor.logger.log(Level.SEVERE, "Can't load event_types.png", e);
		}
	}

	public static EventType getFromId(int id) {
		if (id < 0 || id >= values().length) {
			return null;
		}
		return values()[id];
	}

	public int getId() {
		return this.ordinal();
	}

	public String getName() {
		switch (this) {
			case ENTITY:   return "Entity";
			case WARP:     return "Warp";
			case SCRIPT:   return "Skript";
			case INTERACT: return "Interact";
		}
		return "Unbekannt";
	}

	public String getDescription() {
		switch (this) {
			case ENTITY:   return "Ein Entity, das auf dieser Position steht (z.B. ein NPC)";
			case WARP:     return "Sobald man zu dieser Position geht, wird man zu den angegebenen Coordinaten teleportiert";
			case SCRIPT:   return "Wenn man auf dem Tile steht, wird ein Script ausgeführt.";
			case INTERACT: return "Wenn man dieses Tile anspricht, wird ein Script ausgeführt. (Benötigt BLOCKED Gehdata!)";
		}
		return "Für diesen Type existiert keine Description!";
	}

	public BufferedImage getImage() {
		if (this.imageCache == null) {
			this.imageCache = eventTypesImage.getSubimage(this.ordinal() * 16, 0, 16, 16);
		}
		return this.imageCache;
	}

	public BufferedImage getSelectedImage() {
		if (this.selectedImageCache == null) {
			this.selectedImageCache = eventTypesImage.getSubimage(this.ordinal() * 16, 16, 16, 16);
		}
		return this.selectedImageCache;
	}

	public Event createEventInstance(MapTerrain map, int posX, int posY) {
		switch (this) {
			case ENTITY:   return new EventEntity(map, posX, posY);
			case WARP:     return new EventWarp(map, posX, posY);
			case SCRIPT:   return new EventScript(map, posX, posY);
			case INTERACT: return new EventInteract(map, posX, posY);
		}
		return null;
	}

}
