package de.howaner.MapEditor.map.events;

import de.howaner.MapEditor.map.MapTerrain;
import de.howaner.MapEditor.util.Location;
import de.howaner.Pokemon.saveformat.TagCompound;
import java.awt.Component;
import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.JTextField;

public class EventWarp extends Event {
	private Location target;

	public EventWarp(MapTerrain map, int posX, int posY) {
		this(map, posX, posY, new Location("", 0, 0));
	}

	public EventWarp(MapTerrain map, int posX, int posY, Location target) {
		super(map, posX, posY);
		this.target = target;
	}

	public Location getTarget() {
		return this.target;
	}

	public void setTarget(Location target) {
		this.target = target;
	}

	@Override
	public void loadFromCompound(TagCompound compound) {
		super.loadFromCompound(compound);
		this.target.set(compound.getString("TargetWorld"), compound.getInt("TargetX"), compound.getInt("TargetY"));
	}

	@Override
	public void saveToCompound(TagCompound compound) {
		super.saveToCompound(compound);
		compound.setString("TargetWorld", this.target.getWorld());
		compound.setInt("TargetX", this.target.getPosX());
		compound.setInt("TargetY", this.target.getPosY());
	}

	@Override
	public void addGuiComponents(Box box) {
		box.add(this.createHorizontalBox("Weltname: ", new JTextField(), this.target.getWorld()));
		box.add(this.createHorizontalBox("X Position: ", new JTextField(), String.valueOf(this.target.getPosX())));
		box.add(this.createHorizontalBox("Y Position: ", new JTextField(), String.valueOf(this.target.getPosY())));
	}

	@Override
	public void readFromGuiComponents(Box box) {
		if (box.getComponentCount() < 4) {
			return;
		}

		String worldName = this.readTextFromFieldBox(box.getComponent(box.getComponentCount() - 4));
		String xString = this.readTextFromFieldBox(box.getComponent(box.getComponentCount() - 3));
		String yString = this.readTextFromFieldBox(box.getComponent(box.getComponentCount() - 2));

		this.target.setWorld(worldName);

		try {
			this.target.setPosX(Integer.parseInt(xString));
		} catch (NumberFormatException e) { }

		try {
			this.target.setPosY(Integer.parseInt(yString));
		} catch (NumberFormatException e) { }
	}

	@Override
	public EventType getType() {
		return EventType.WARP;
	}

	public String readTextFromFieldBox(Component component) {
		Box box = (Box) component;
		return ((JTextField) box.getComponent(2)).getText();
	}

	private Box createHorizontalBox(String name, JTextField field, String defaultText) {
		if (defaultText != null)
			field.setText(defaultText);

		Box box = Box.createHorizontalBox();
		box.add(new JLabel(name));
		box.add(Box.createHorizontalGlue());
		box.add(field);
		return box;
	}

}
