package de.howaner.MapEditor.map.events;

import de.howaner.MapEditor.map.MapTerrain;
import de.howaner.Pokemon.saveformat.TagBase;
import de.howaner.Pokemon.saveformat.TagCompound;
import java.util.List;

public class MapEvents {
	private final MapTerrain map;
	private Event[] events;

	public MapEvents(MapTerrain map) {
		this.map = map;
		this.events = new Event[map.getWidth() * map.getHeight()];
	}

	public Event[] getDataArray() {
		return this.events;
	}

	public Event getEventAt(int x, int y) {
		return this.events[y * this.map.getWidth() + x];
	}

	public void setEvent(int x, int y, Event event) {
		this.events[y * this.map.getWidth() + x] = event;
	}

	public void loadFromTagList(List<TagBase> list) {
		this.events = new Event[map.getWidth() * map.getHeight()];

		for (TagBase tag : list) {
			TagCompound compound = (TagCompound) tag;
			EventType type = EventType.getFromId(compound.getInt("Type"));

			Event event = type.createEventInstance(this.map, 0, 0);
			event.loadFromCompound(compound);

			this.setEvent(event.getPosition().getPosX(), event.getPosition().getPosY(), event);
		}
	}

	public void saveToTagList(List<TagBase> list) {
		for (Event event : this.events) {
			if (event == null) {
				continue;
			}

			TagCompound compound = new TagCompound();
			compound.setInt("Type", event.getType().getId());
			event.saveToCompound(compound);

			list.add(compound);
		}
	}

}
