package de.howaner.MapEditor.map.tileset;

import de.howaner.Pokemon.saveformat.TagCompound;
import java.awt.image.BufferedImage;

public class TileAnimation {
	private TileAnimationType animationType = null;
	private BufferedImage animationImage = null;
	private float animationDuration = 0.10f;
	private float animationDelay = 1f;

	private String particleName = "";
	private float particleDuration = 0.25f;

	private boolean hidePlayerFoot = false;

	public TileAnimationType getAnimationType() {
		return this.animationType;
	}

	public BufferedImage getAnimationImage() {
		return this.animationImage;
	}

	public float getAnimationDuration() {
		return this.animationDuration;
	}

	public float getAnimationDelay() {
		return this.animationDelay;
	}

	public String getParticleName() {
		return this.particleName;
	}

	public float getParticleDuration() {
		return this.particleDuration;
	}

	public boolean getHidePlayerFoot() {
		return this.hidePlayerFoot;
	}

	public void setAnimationType(TileAnimationType type) {
		this.animationType = type;
	}

	public void setAnimationImage(BufferedImage image) {
		this.animationImage = image;
	}

	public void setAnimationDuration(float duration) {
		this.animationDuration = duration;
	}

	public void setAnimationDelay(float delay) {
		this.animationDelay = delay;
	}

	public void setParticleName(String particleName) {
		this.particleName = particleName;
	}

	public void setParticleDuration(float duration) {
		this.particleDuration = duration;
	}

	public void setHidePlayerFoot(boolean hidePlayerFoot) {
		this.hidePlayerFoot = hidePlayerFoot;
	}

	public boolean isEmpty() {
		return (this.animationType == null && this.particleName.isEmpty() && !this.hidePlayerFoot);
	}

	public void loadFromCompound(TagCompound compound) throws Exception {
		if (compound.containsTag("AnimationType")) {
			this.animationType = TileAnimationType.valueOf(compound.getString("AnimationType"));
			this.animationImage = MapTileset.decodeBase64(compound.getString("AnimationImage"));
			this.animationDuration = compound.getFloat("AnimationDuration");
			this.animationDelay = compound.getFloat("AnimationDelay");
		}

		if (compound.containsTag("ParticleName")) {
			this.particleName = compound.getString("ParticleName");
			this.particleDuration = compound.getFloat("ParticleDuration");
		}

		if (compound.containsTag("HidePlayerFoot")) {
			this.hidePlayerFoot = compound.getBoolean("HidePlayerFoot");
		}
	}

	public void saveToCompound(TagCompound compound) throws Exception {
		if (this.animationType != null) {
			compound.setString("AnimationType", this.animationType.name());
			compound.setString("AnimationImage", MapTileset.encodeBase64(this.animationImage));
			compound.setFloat("AnimationDuration", this.animationDuration);
			compound.setFloat("AnimationDelay", this.animationDelay);
		}

		if (!this.particleName.isEmpty()) {
			compound.setString("ParticleName", this.particleName);
			compound.setFloat("ParticleDuration", this.particleDuration);
		}

		compound.setBoolean("HidePlayerFoot", this.hidePlayerFoot);
	}

	public enum TileAnimationType {
		NORMAL, WALK, STAND;
	}

}
