package de.howaner.MapEditor.map.tileset;

import de.howaner.MapEditor.util.MathHelper;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;

public class TilesetConverter {
	
	public static MapTileset convertFileToTileset(File file) throws IOException {
		BufferedImage image = ImageIO.read(file);
		return convertImageToTileset(image);
	}

	public static void writeTilesetToFile(MapTileset tileset, File file) throws IOException {
		if (file.exists()) {
			file.delete();
		}
		
		BufferedImage image = convertTilesetToImage(tileset);
		ImageIO.write(image, "png", file);
	}

	public static BufferedImage convertTilesetToImage(MapTileset tileset) {
		if (tileset.getTiles().isEmpty()) {
			return new BufferedImage(16 * 16, 4 * 16, BufferedImage.TYPE_INT_ARGB);
		}

		BufferedImage image = new BufferedImage(
			Math.min(16, tileset.getTiles().size()) * 16,
			MathHelper.roundUp(tileset.getTiles().size() / 16) * 16,
			BufferedImage.TYPE_INT_ARGB
		);

		int xCoord = 0;
		int yCoord = 0;
		for (BufferedImage tile : tileset.getTiles()) {
			// Copy the complete tile (16 pixels) to the image:
			for (int x = 0; x < 16; x++) {
				for (int y = 0; y < 16; y++) {
					image.setRGB(xCoord + x, yCoord + y, tile.getRGB(x, y));
				}
			}
			
			xCoord += 16;
			if (xCoord >= 16 * 16) {
				xCoord = 0;
				yCoord += 16;
			}
		}

		return image;
	}

	public static BufferedImage convertTilesetToImageWithGrid(MapTileset tileset) {
		if (tileset.getTiles().isEmpty()) {
			return new BufferedImage(16 * 16, 4 * 16, BufferedImage.TYPE_INT_ARGB);
		}

		BufferedImage image = new BufferedImage(
			Math.min(16, tileset.getTiles().size()) * 17 - 1,
			MathHelper.roundUp(tileset.getTiles().size() / 16) * 17 - 1,
			BufferedImage.TYPE_INT_ARGB
		);

		int xCoord = 0;
		int yCoord = 0;
		for (BufferedImage tile : tileset.getTiles()) {
			// Copy the complete tile (16 pixels) to the image:
			for (int x = 0; x < 16; x++) {
				for (int y = 0; y < 16; y++) {
					image.setRGB(xCoord + x, yCoord + y, tile.getRGB(x, y));
				}
			}
			
			xCoord += 17;
			if (xCoord >= 16 * 17) {
				xCoord = 0;
				yCoord += 17;
			} else {
				for (int y = 0; y < 16; y++) {
					image.setRGB(xCoord, yCoord + y, 16777215);
				}
			}
		}

		return image;
	}

	public static MapTileset convertImageToTileset(BufferedImage image) {
		MapTileset tileset = new MapTileset();
		tileset.setTilesPerLine(image.getWidth() / 16);

		for (int y = 0; y < image.getHeight(); y += 16) {
			for (int x = 0; x < image.getWidth(); x += 16) {
				BufferedImage tile = image.getSubimage(x, y, 16, 16);
				tileset.addTile(tile);
			}
		}

		return tileset;
	}

}
